require_relative "./core"

module Sekreti
  # Access helper to call the encryption and decryption
  # actions.
  class Crypt
    def initialize()
      @options = {
        path: nil,
        output_file: nil,
        protocol: "AES-128-CBC",
        status: false
      }
    end

    # Returns encryption protocol
    # @return encryption protocol
    def protocol
      @options[:protocol]
    end

    def protocol?
      unless @options.key?(:protocol)
        return false
      end

      true
    end

    def protocol=(protocol)
      @options[:protocol] = protocol
    end

    # Returns entry file
    # @return entry file
    def file
      @options[:path]
    end

    # Defines entry file
    # @param path [String] relative or absolute path to 
    #   entry file.
    def file=(path)
      @options[:path] = path
    end

    # Returns true if entry file is defined and
    # exists. 
    # @return boolean
    def file?
      unless @options.key?(:path)
        return false
      end

      unless File.file? @options[:path]
        return false
      end

      true
    end

    # Returns output file parameter
    # @return output file parameter
    def output_file
      @options[:output_file]
    end

    # Defines the output file
    # @param output_file [String] relative or absolute path
    #   to output file.
    def output_file=(output_file)
      @options[:output_file] = output_file
    end

    # Returns true if the output file is defined and 
    #  it doesn't exist yet.
    # @return bool
    def output_file?
      unless @options.key?(:output_file)
        return false
      end

      if File.file? @options[:output_file]
        return false
      end

      true
    end

    # Returns the encryption key
    # @return encryption key
    def key
      @options[:key]
    end

    # Set the encryption key
    # @param key [String] 16 bytes string
    # @return boolean
    def key=(key)
      if @options[:protocol] == "AES-128-CBC"
        raise(StandardError, "Key must be 16 bytes long.") if key.length != 16
      end

      if @options[:protocol] == "AES-256-CBC"
        raise(StandardError, "Key must be 32 bytes long.") if key.length != 32
      end

      @options[:key] = key
    end

    # returns true if key is defined.
    # @return boolean
    def key?
      unless @options.key?(:key)
        return false
      end

      true
    end

    # Encrypts a file with parameters set
    # @see Core.encrypt!
    def encrypt!
      raise StandardError, "No entry file defined." unless file?
      raise StandardError, "No output file defnined" unless output_file?
      raise StandardError, "No key defined" unless key?

      if Core.encrypt!(@options)
        @options[:status] = true
      else
        @options[:status] = false
      end
    end

    # Decrypts a file with parameters set
    # @see Core.decrypt!
    def decrypt!
      raise StandardError, "No entry file defined." unless file?
      raise StandardError, "No output file defnined" unless output_file?
      raise StandardError, "No key defined" unless key?

      if Core.decrypt!(@options)
        @options[:status] = true
      else
        @options[:status] = false
      end
    end

    # Returns the action status.
    def status?
      @options[:status]
    end

    # Returns the configuration
    # @return current instance configuration.
    def dump
      @options
    end
  end
end